// Clarida, Gali, Gertler model

var da pie r rstar tau u x; 	  

varexo eps_u eps_a eps_tau;   

parameters phi beta kappa phi_x phi_pie alpha rho lambda delta; // DECLARATION OF THE DEEP PARAMETERS.

// Parameter Values  
beta    = 0.99;
phi_x   = 0.;
phi_pie = 1.5;
alpha   = 0;
rho     = 0.2;
lambda  = 0.5;
delta   = 0.2;
phi     = 1;
theta   = 0.75;
kappa   = ((1-theta)*(1-beta*theta)*(1+phi))/(theta);

// DECLARATION OF THE (LINEAR) DSGE MODEL: 
model(linear);

    beta*pie(+1) + kappa*x = pie;                                      // Calvo Pricing Equation (Eqn 1)
    r - pie(+1)-rstar   = x(+1) - x;                                   // Intertemporal Equation (Eqn 2)
    alpha*r(-1) + u + (1-alpha)*phi_pie*pie + (1-alpha)*phi_x*x = r;   // Monetary Policy Rule (Eqn 3)
    rstar = rho*da + (1-lambda)/(1+phi)*tau;                           // Definition of the Natural Rate (Eqn 4)
    da = rho*da(-1) + eps_a;                                           // Exogenous process for da (Eqn 5)
    tau       = lambda*tau(-1) + eps_tau;                              // Exogenous process for tau (Eqn 6)
    u         = delta*u(-1) + eps_u;                                   // Exogenous process for u (Eqn 7)
end;


shocks;
var eps_a;
stderr .01;
var eps_tau;
stderr .01;
var eps_u;
stderr .000001;
end;

steady;
check;


//dbstop in stoch_simul at 22;
stoch_simul(irf=7,periods=5000,nograph) da pie r rstar tau u x; 

ConditionalForecastStochastic;
return;

figure;
subplot(231)
plot(cumsum(da_eps_a))
title('level of technology')
axis tight
subplot(232)
plot(pie_eps_a)
title('inflation')
axis tight
subplot(233)
plot(r_eps_a)
title('nominal rate of interest')
axis tight
subplot(234)
plot(rstar_eps_a)
title('natural rate of interest')
axis tight
subplot(235)
plot(x_eps_a-tau_eps_a/(1+phi))
title('hours worked')
axis tight
subplot(236)
plot(x_eps_a+cumsum(da_eps_a))
title('output')
axis tight









